@@ -18,7 +18,7 @@ class InProcessChannel:
     proxy_methods: List[object] = []
 
     def __init__(self, client=None):
-        """Initialze the channel."""
+        """Initialize the channel."""
         super().__init__()
         self.client = client
         self._is_alive = False
@@ -40,7 +40,8 @@ class InProcessChannel:
 
         Subclasses should override this method to handle incoming messages.
         """
-        raise NotImplementedError("call_handlers must be defined in a subclass.")
+        msg = "call_handlers must be defined in a subclass."
+        raise NotImplementedError(msg)
 
     def flush(self, timeout=1.0):
         """Flush the channel."""
