@@ -3,6 +3,8 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+from __future__ import annotations
+
 import errno
 import json
 import os
@@ -10,10 +12,15 @@ import shutil
 import stat
 import sys
 import tempfile
+from typing import Any
 
 from jupyter_client.kernelspec import KernelSpecManager
+from traitlets import Unicode
 
-from .debugger import _is_debugpy_available
+try:
+    from .debugger import _is_debugpy_available
+except ImportError:
+    _is_debugpy_available = False
 
 pjoin = os.path.join
 
@@ -23,7 +30,11 @@ KERNEL_NAME = "python%i" % sys.version_info[0]
 RESOURCES = pjoin(os.path.dirname(__file__), "resources")
 
 
-def make_ipkernel_cmd(mod="ipykernel_launcher", executable=None, extra_arguments=None):
+def make_ipkernel_cmd(
+    mod: str = "ipykernel_launcher",
+    executable: str | None = None,
+    extra_arguments: list[str] | None = None,
+) -> list[str]:
     """Build Popen command list for launching an IPython kernel.
 
     Parameters
@@ -48,7 +59,7 @@ def make_ipkernel_cmd(mod="ipykernel_launcher", executable=None, extra_arguments
     return arguments
 
 
-def get_kernel_dict(extra_arguments=None):
+def get_kernel_dict(extra_arguments: list[str] | None = None) -> dict[str, Any]:
     """Construct dict for kernel.json"""
     return {
         "argv": make_ipkernel_cmd(extra_arguments=extra_arguments),
@@ -58,7 +69,11 @@ def get_kernel_dict(extra_arguments=None):
     }
 
 
-def write_kernel_spec(path=None, overrides=None, extra_arguments=None):
+def write_kernel_spec(
+    path: str | None = None,
+    overrides: dict[str, Any] | None = None,
+    extra_arguments: list[str] | None = None,
+) -> str:
     """Write a kernel spec directory to `path`
 
     If `path` is not specified, a temporary directory is created.
@@ -89,14 +104,14 @@ def write_kernel_spec(path=None, overrides=None, extra_arguments=None):
 
 
 def install(
-    kernel_spec_manager=None,
-    user=False,
-    kernel_name=KERNEL_NAME,
-    display_name=None,
-    prefix=None,
-    profile=None,
-    env=None,
-):
+    kernel_spec_manager: KernelSpecManager | None = None,
+    user: bool = False,
+    kernel_name: str = KERNEL_NAME,
+    display_name: str | None = None,
+    prefix: str | None = None,
+    profile: str | None = None,
+    env: dict[str, str] | None = None,
+) -> str:
     """Install the IPython kernelspec for Jupyter
 
     Parameters
@@ -132,7 +147,7 @@ def install(
         # kernel_name is specified and display_name is not
         # default display_name to kernel_name
         display_name = kernel_name
-    overrides = {}
+    overrides: dict[str, Any] = {}
     if display_name:
         overrides["display_name"] = display_name
     if profile:
@@ -150,7 +165,7 @@ def install(
     )
     # cleanup afterward
     shutil.rmtree(path)
-    return dest
+    return dest  # type:ignore[no-any-return]
 
 
 # Entrypoint
@@ -161,14 +176,16 @@ from traitlets.config import Application
 class InstallIPythonKernelSpecApp(Application):
     """Dummy app wrapping argparse"""
 
-    name = "ipython-kernel-install"
+    name = Unicode("ipython-kernel-install")
 
-    def initialize(self, argv=None):
+    def initialize(self, argv: list[str] | None = None) -> None:
+        """Initialize the app."""
         if argv is None:
             argv = sys.argv[1:]
         self.argv = argv
 
-    def start(self):
+    def start(self) -> None:
+        """Start the app."""
         import argparse
 
         parser = argparse.ArgumentParser(
@@ -221,7 +238,7 @@ class InstallIPythonKernelSpecApp(Application):
         )
         opts = parser.parse_args(self.argv)
         if opts.env:
-            opts.env = {k: v for (k, v) in opts.env}
+            opts.env = dict(opts.env)
         try:
             dest = install(
                 user=opts.user,
@@ -238,7 +255,7 @@ class InstallIPythonKernelSpecApp(Application):
                     print("Perhaps you want `sudo` or `--user`?", file=sys.stderr)
                 self.exit(1)
             raise
-        print("Installed kernelspec %s in %s" % (opts.name, dest))
+        print(f"Installed kernelspec {opts.name} in {dest}")
 
 
 if __name__ == "__main__":
